#!/usr/bin/python3
import pymysql
from notetool.secret import read_secret

host = read_secret(cate1="database", cate2="mysql", cate3="host")
user = read_secret(cate1="database", cate2="mysql", cate3="user")
password = read_secret(cate1="database", cate2="mysql", cate3="password")
database = read_secret(cate1="database", cate2="mysql", cate3="database")


def create_database(database=None):
    # 打开数据库连接
    print(host, user, password, database)
    db = pymysql.connect(host=host, user=user, password=password)

    # 使用 cursor() 方法创建一个游标对象 cursor
    cursor = db.cursor()

    cursor.execute(f'CREATE DATABASE {database};')

    # 使用 execute()  方法执行 SQL 查询
    cursor.execute("show databases")

    data = cursor.fetchmany()

    print(f"Database {data}")

    # 关闭数据库连接
    db.close()


def create_table(database=None):
    # 打开数据库连接
    print(host, user, password, database)
    db = pymysql.connect(host=host, user=user, password=password, database=database)

    # 使用 cursor() 方法创建一个游标对象 cursor
    cursor = db.cursor()

    # 使用 execute()  方法执行 SQL 查询
    cursor.execute("show tables")

    data = cursor.fetchmany()

    print(f"Tables {data}")

    # 关闭数据库连接
    db.close()


create_table(database='notecoin')



import logging
import os

from notetool.secret import read_secret
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker

uri = read_secret(cate1='notecoin', cate2='dataset', cate3='db_path')
uri = uri or f'sqlite:///{os.path.abspath(os.path.dirname(__file__))}/data/notecoin.db'
engine = create_engine(uri, echo=True)
Base = declarative_base()
session = sessionmaker(bind=engine)()

logging.info(f'uri:{uri}')

def create_all():
    Base.metadata.create_all(engine)


import base64
import hmac
import json
import logging
import time
from typing import List

from notecoin.okex.websocket.handle import BaseHandle
from notecoin.okex.websocket.utils import get_local_timestamp

from websocket import WebSocket, WebSocketException, create_connection


class BaseConnect:
    def __init__(self, url, channels, api_key=None, secret_key=None, passphrase=None, private=False):
        self.url = url
        self.channels = channels
        self.api_key = api_key
        self.secret_key = secret_key
        self.passphrase = passphrase
        self.private = private
        self.ws: WebSocket = create_connection(self.url)
        self.handles: List[BaseHandle] = []

    def run(self):
        self.subscribe_start()
        while True:
            try:
                res = self.ws.recv()
            except (TimeoutError, WebSocketException):
                try:
                    self.ping()
                except Exception as e:
                    logging.warning(f"连接关闭，正在重连:{e}")
                    self.subscribe_restart()
                continue
            self.handle_data(res)

    def handle_data(self, res):
        for handle in self.handles:
            handle.solve(res)

    def ping(self):
        self.ws.send('ping')
        res = self.ws.recv()
        logging.info(res)
        assert res == 'pong'

    def subscribe_restart(self):
        self.subscribe_stop()
        self.subscribe_start()

    def subscribe_start(self):
        self.ws: WebSocket = create_connection(self.url)
        if self.private:
            timestamp = str(get_local_timestamp())
            message = timestamp + 'GET' + '/users/self/verify'
            mac = hmac.new(bytes(self.secret_key, encoding='utf8'), bytes(message, encoding='utf-8'),
                           digestmod='sha256')
            sign = base64.b64encode(mac.digest()).decode("utf-8")
            login_param = {"op": "login", "args": [{"apiKey": self.api_key,
                                                    "passphrase": self.passphrase,
                                                    "timestamp": timestamp,
                                                    "sign": sign}]}
            login_str = json.dumps(login_param)
        else:
            login_str = json.dumps({"op": "subscribe", "args": self.channels})

        self.ws.send(login_str)
        logging.info(f"send: {login_str}")
        res = self.ws.recv()
        logging.info(f"recv: {res}")
        time.sleep(1)

    def subscribe_stop(self):
        self.ws: WebSocket = create_connection(self.url)
        sub_param = {"op": "unsubscribe", "args": self.channels}
        sub_str = json.dumps(sub_param)
        self.ws.send(sub_str)
        logging.info(f"send: {sub_str}")
        res = self.ws.recv()
        logging.info(f"recv: {res}")


class PublicConnect(BaseConnect):
    def __init__(self, channels, *args, **kwargs):
        super(PublicConnect, self).__init__(url="wss://ws.okx.com:8443/ws/v5/public",
                                            channels=channels, *args, **kwargs)


class PrivateConnect(BaseConnect):

    def __init__(self, channels, *args, **kwargs):
        super(PrivateConnect, self).__init__(url='wss://ws.okx.com:8443/ws/v5/private', private=True,
                                             channels=channels, *args, **kwargs)


class TradeConnect(PrivateConnect):
    def __init__(self, *args, **kwargs):
        super(TradeConnect, self).__init__(*args, **kwargs)



import logging

from notecoin.database.base import create_all, session
from notecoin.okex.websocket.dataset import OkexSocketPublicTickers
from notecoin.okex.websocket.utils import get_local_timestamp


class BaseHandle:
    def __init__(self, channels, *args, **kwargs):
        self.channels = channels

    def solve(self, data) -> bool:
        data = eval(data)
        print(f"{get_local_timestamp()}\t{len(data)}")
        if 'event' in data:
            return False
        channel = data['args']['channel']
        if channel in self.channels:
            return self.handle(data)

    def handle(self, data) -> bool:
        print(f"{data['args']['channel']}:{len(str(data))}")
        return True


class PublicTickers(BaseHandle):
    def __init__(self, *args, **kwargs):
        create_all()
        super(PublicTickers, self).__init__(channels=['tickers'], *args, **kwargs)

    def handle(self, data) -> bool:
        try:
            for arg in data['data']:
                session.merge(OkexSocketPublicTickers(**arg))
                session.commit()
        except Exception as e:
            logging.error(f"error:{e}")
            return False
        return True
